import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/* 
 * Sotto-classe di Programma, realizza l'interfaccia grafica per
 * un utente gestore.
 * Ha tre finestre: finestra iniziale di scelta, finestra di ricerca,
 * dei risultati della ricerca, di creazione di un nuovo abbonato.
 * Tutti gli attributi e metodi sono di classe, non e' necessario
 * creare un oggetto di classe Programma.
 */
public class ProgrammaGestore extends Programma
{
  /** Finestra iniziale per scelta dell'operazione da fare. */
  protected static Finestra finScelta;
  /** Pannello da mettere al centro della finestra iniziale di scelta. */
  protected static PannelloScelta panScelta;
  /** Finestra per inserire un nuovo abbonato. */
  protected static Finestra finNuovo;
  /** Pannello da mettere al centro della finestra di nuovo abbonato. */
  protected static PannelloNuovo panNuovo;
  /** Finestra per specificare il criterio di ricerca. */
  protected static Finestra finCriterio;
  /** Pannello da mettere al centro della finestra del criterio di ricerca. */  
  protected static PannelloCriterio panCriterio;
  /** Finestra per mostrare il risultato della ricerca. */
  protected static Finestra finRisultato;
  /** Pannello da mettere al centro della finestra che mostra il risultato. */
  protected static PannelloRisultato panRisultato;

  /** Indice del risultato correntemente selezionato per essere mostrato. */
  protected static int indice;
  
  /** Ereditato dalla super-classe, il risultato sara' non un utente          
    * generale di classe Utente, ma di classe UtenteGestore. */
  public static UtenteGestore getUtente()
  {  return (UtenteGestore)(utente);  }

  /**
   * Costruisce le quattro finestre di scelta iniziale, di nuovo abbonato,
   * del criterio di ricerca, e di visione dei risultati.
   * mostra la finestra di scelta iniziale.
   */
  protected static void prepara()
  {
    // Costruisce finestra di scelta iniziale

    finScelta = new Finestra(null);
    finScelta.setTitle("Benvenuto gestore del data base abbonati");
    panScelta = new PannelloScelta();
    finScelta.mettiInCentro(panScelta);
    finScelta.mettiInBasso(panScelta.btCerca);
    finScelta.mettiInBasso(panScelta.btNuovo);
    finScelta.pack();

    // Callback per bottone chiudi        
    finScelta.btChiudi.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  tentaUscita(finScelta,null);  }
    });
        
    // Callback del bottone ricerca nella finestra di scelta.
    // Fa partire altra finestra finCriterio che chiedera' parametri
    // di ricerca
    panScelta.btCerca.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        panCriterio.azzeraInfo();
        finCriterio.setVisible(true);
      }
    });

    // Callback del bottone nuovo abbonato nella finestra di scelta.
    // Fa partire altra finestra finNuovo che chiedera' i dati.
    panScelta.btNuovo.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        panNuovo.azzeraInfo();
        finNuovo.setVisible(true);
      }
    });
    
    // Costruisce finestra nuovo abbonato

    finNuovo = new Finestra(finScelta);
    finNuovo.setTitle("Crea nuovo abbonato");
    panNuovo = new PannelloNuovo();
    finNuovo.mettiInCentro(panNuovo);
    finNuovo.mettiInBasso(panNuovo.btSalva);
    finNuovo.mettiInBasso(panNuovo.btAzzera);
    finNuovo.pack();

    // Callback per bottone chiudi
    finNuovo.btChiudi.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  tentaUscita(finNuovo,panNuovo);  }
    });

    // Callback per bottone salva
    panNuovo.btSalva.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        try
        {
          InfoBase aux = panNuovo.prendiInfo(true);
          utente.aggiorna(aux);
          salvaAbbonatoCorrente(finNuovo,panNuovo);
          finNuovo.setVisible(false);
        }
        catch (Exception excp)
        {
          JOptionPane.showMessageDialog(finCriterio,excp.getMessage(),"",
                      JOptionPane.ERROR_MESSAGE);
        }        
      }
    });

    // Callback per bottone azzera
    panNuovo.btAzzera.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  panNuovo.azzeraInfo();  }
    });

    // Costruisce finestra di ricerca

    finCriterio = new Finestra(finScelta);
    finCriterio.setTitle("Ricerca nel data base abbonati");
    panCriterio = new PannelloCriterio();
    finCriterio.mettiInCentro(panCriterio);
    finCriterio.mettiInBasso(panCriterio.btCerca);
    finCriterio.mettiInBasso(panCriterio.btAzzera);
    finCriterio.pack();

    // Callback per bottone chiudi (in tentaUscita il pannello e' null
    // e non panCriterio, perche' altrimenti cerca di salvare i dati) 
    finCriterio.btChiudi.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  tentaUscita(finCriterio,null);  }
    });
    
    // Callback per bottone cerca
    panCriterio.btCerca.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        try
        {
          Criterio cr = new Criterio(panCriterio.prendiInfo(false));
          getUtente().setCriterio(cr);
          getUtente().ricerca();
          if (getUtente().numeroRisultati()==0)
               JOptionPane.showMessageDialog(finCriterio,"Nessun risultato");
          else
          {
            mostraRisultato(0);
            finRisultato.setVisible(true);
          }
        }
        catch (Exception excp)
        {
          JOptionPane.showMessageDialog(finCriterio,excp.getMessage(),"",
                      JOptionPane.ERROR_MESSAGE);
        }
      }
    });

    // Callback per bottone azzera
    panCriterio.btAzzera.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  panCriterio.azzeraInfo();  }
    });

    // Costruisce finestra risultato della ricerca

    finRisultato = new Finestra(finCriterio);
    finRisultato.setTitle("Abbonati trovati nella ricerca");
    panRisultato = new PannelloRisultato();
    finRisultato.mettiInCentro(panRisultato);
    finRisultato.mettiInBasso(panRisultato.btSalva);
    finRisultato.mettiInBasso(panRisultato.btRicarica);
    finRisultato.mettiInBasso(panRisultato.btIndietro);
    finRisultato.mettiInBasso(panRisultato.btAvanti);
    finRisultato.pack();

    // Callback per bottone chiudi        
    finRisultato.btChiudi.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  tentaUscita(finRisultato,panRisultato);  }
    });

    // Callback per bottone salva
    panRisultato.btSalva.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        try
        {
          InfoBase aux = panRisultato.prendiInfo(true);
          utente.aggiorna(aux);
          if (!utente.haModificato())
             JOptionPane.showMessageDialog(finRisultato,
                         "Abbonato non modificato, non necessario salvare"); 
          else
             salvaAbbonatoCorrente(finRisultato,panRisultato);
        }
        catch (Exception excp)
        {
          JOptionPane.showMessageDialog(finCriterio,excp.getMessage(),"",
                      JOptionPane.ERROR_MESSAGE);
        }        
      }
    });

    // Callback per bottone ricarica
    panRisultato.btRicarica.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {
        ricaricaAbbonatoCorrente(finRisultato,panRisultato);
      }
    });

    // Callback per bottone indietro    
    panRisultato.btIndietro.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  mostraRisultato(indice-1);  }
    });

    // Callback per bottone avanti
    panRisultato.btAvanti.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
      {  mostraRisultato(indice+1);  }
    });
  }

  /** Seleziona nell'utente e mostra nel pannello risultati l'i-esimo
   * abbonamento risultato dalla ricerca. */
  protected static void mostraRisultato(int i)
  {
     indice = i;
     getUtente().selezionaRisultato(indice);
     panRisultato.mettiInfo(utente.copiaSelezionato());
     if (i==0) panRisultato.btIndietro.setEnabled(false);
     else panRisultato.btIndietro.setEnabled(true);
     if (i==getUtente().numeroRisultati()-1)
          panRisultato.btAvanti.setEnabled(false);  
     else panRisultato.btAvanti.setEnabled(true);
  }

  public static void main(String[] args)
  {
    if (args.length!=1) 
    {  System.out.println("Specificare il nome della directory");
    }
    else
    try
    {
      utente = new UtenteGestore(args[0]);
      prepara();
      finScelta.setVisible(true);
    }
    catch (Exception  excp) 
    {
      JOptionPane.showMessageDialog(finScelta,excp.getMessage(),"",
                      JOptionPane.ERROR_MESSAGE);
      excp.printStackTrace();
    }
  }
  
}

